@@ -16,7 +16,7 @@ B_model = training_model(dat = UCICreditCard,
                         missing_proc = TRUE,
                         outlier_proc = TRUE,
                         trans_log = TRUE,
-                        feature_filter = list(filter = c("IV", "PSI", "COR", "XGB"),
+                        feature_filter = list(filter = c("IV", "COR"),
                                             cv_folds = 1,
                                             iv_cp = 0.02,
                                             psi_cp = 0.2,
@@ -24,7 +24,7 @@ B_model = training_model(dat = UCICreditCard,
                                             xgb_cp = 0,
                                             hopper = TRUE),
                         vars_plot = FALSE,
-                        algorithm = list("LR","XGB"),
+                        algorithm = list("LR"),
                         breaks_list = NULL,
                         LR.params = lr_params(
                             iter = 2,
@@ -45,18 +45,6 @@ B_model = training_model(dat = UCICreditCard,
                             f_eval = 'ks',
                             lasso = TRUE,
                             step_wise = FALSE),
-                        XGB.params = xgb_params(
-                            iter = 3,
-                            method = 'random_search',
-                            params = list(
-                                max_depth = c(3:6),
-                                eta = c(0.01, 0.05, 0.1, 0.2),
-                                gamma = c(0.01, 0.05, 0.1),
-                                min_child_weight = c(1, 5, 10, 20, 30, 40, 50),
-                                subsample = c(0.8, 0.7, 0.6, 0.5),
-                                colsample_bytree = c(0.8, 0.7, 0.6, 0.5),
-                                scale_pos_weight = c(1, 2, 3)),
-                            f_eval = 'auc'),
                         parallel = FALSE,
                         cores_num = NULL,
                         save_pmml = FALSE,
